'PADS PowerPCB script - Select Unrouted SMD Pins ver1.01
'prgm_nam:    Select_SMDUnrouted.BAS
'codebase:    PADS PowerPCB VB (SAX)
'PPCB ver:    5.0x - 2007
'created:     11.04.09
'author:      Ron Olinyk, Mindlink Technologies | rono@mindlinktech.com | www.mindlinktech.com
'description: Highlights SMD pins missing routes for selected (or all) nets in design. 
'             Note: Although it does not find completed fanouts ie. routed stubs terminating in a via,
'                   it can be used for indicating possible missing fanouts (SMD pins without attached routes).
'
'Quick Run Notes
' 1. Select a net or nets (must be a *net* selection, not a pin-pair or segment). If nothing is selected, the
'    program will process all nets in design.
' 2. Run script. If you use Tools | Basic Scripts | Basic Scripts (with script file already loaded) you can 
'    repeatedly re-run script while analyzing/completing fanouts.
' 3. Any unrouted SMD pins on selected nets will be highlighted. 
'    The bottom status bar will report results.

'==============================================================================

' Global collections
Dim colrseg As Object      
Dim colusrsmd As Object    
Dim colselnet As Object    
Dim colunroutd As Object

Sub Main

    Dim xnet As Object
    Dim msg As String
    Dim cnt As Integer
    
    ' Intialize global collections
    Set colusrsmd = ActiveDocument.GetObjects(0)
    Set colrseg = ActiveDocument.GetObjects(0)
    Set colselnet = ActiveDocument.GetObjects(0)
    Set colunroutd = ActiveDocument.GetObjects(0)
    
    ' get collection of all selected nets
    Set colselnet = ActiveDocument.GetObjects(ppcbObjectTypeNet,,True)
    msg = "selected "
    If colselnet.Count =0 Then
       Set colselnet = ActiveDocument.GetObjects(ppcbObjectTypeNet,,)
       msg = ""
    End If
    
    For Each xnet In colselnet
	  NetSMDCollection xnet
	  NetSegCollection xnet
	  UnroutedPins
	  
	Next xnet  
	
    ActiveDocument.SelectObjects(,,False)
    ' select unrouted pins
    colunroutd.Select
    
    cnt = colunroutd.Count
    If cnt = 0 Then
      StatusBarText = "All " & msg & "net SMD pins routed"
    Else
      StatusBarText = cnt & " " & msg & "net SMD pins unrouted"
    End If  

End Sub

' Get collection of SMD pins for selected net
' Object collection is defined as Global

Private Sub NetSMDCollection (ByVal xnet As Object)
 Dim xpin As Object

  colusrsmd.Reset
  
  For Each xpin In xnet.Pins             
       If xpin.IsSMD Then                
         colusrsmd.Add (xpin)           
       End If
  Next xpin


End Sub

' Get collection of route segments for selected net
' Only collect segments at component layers (top, bottom)
' Object collection is defined as Global

Private Sub NetSegCollection (ByVal xnet As Object)
 Dim xseg As Object
 Dim xconn As Object
 Dim sx As Integer
    
   colrseg.Reset 
   For Each xconn In xnet.Connections
     For Each xseg In xconn.RouteSegments
       If ActiveDocument.LayerType(xseg.layer) = ppcbLayerComponent Then  
        colrseg.Add(xseg)
       End If 
     Next xseg
   Next xconn
         
    
    colrseg.Sort      
    For sx = 1 To colrseg.Count
      If sx = colrseg.Count Then  
        Exit For
      End If
     
      If colrseg(sx) = colrseg(sx+1) Then
        colrseg.Remove(sx)         
        sx = sx - 1                 
      End If
    Next sx 

End Sub

' Get collection of unrouted pins for selected net
' Object collection is defined as Global

Private Sub UnroutedPins
  Dim xseg As Object
  Dim xpin As Object
  
  
  For Each xseg In colrseg
    For Each xpin In colusrsmd
      If xseg.Points(1,1)= xpin.PositionX And xseg.Points(1,2)= xpin.PositionY Then
        colusrsmd.Remove (xpin)
      ElseIf xseg.Points(2,1)= xpin.PositionX And xseg.Points(2,2)= xpin.PositionY Then
        colusrsmd.Remove (xpin)
      End If
  
    Next xpin
  Next xseg
  
  ' Copy unrouted pins to select collection
  For Each xpin In colusrsmd
    colunroutd.Add (xpin)
  Next xpin  
  

  
End Sub


